import Index, { IndexField, ObjectField } from './index';
import JSONStore from './json-store';
import { Query } from './query';
declare class DatabaseIterableIterator<T> implements AsyncIterableIterator<T> {
    protected iterator: AsyncIterableIterator<[number, T]>;
    constructor(iterator: AsyncIterableIterator<[number, T]>);
    next(): Promise<IteratorResult<T>>;
    toArray(): Promise<T[]>;
    [Symbol.asyncIterator](): this;
}
declare class Database<T extends Record = Record> {
    protected store: JSONStore<T>;
    protected _index: Index;
    protected logger: Console;
    constructor(filename: string);
    create(indexFields?: (string | IndexField)[]): Promise<void>;
    drop(): Promise<void>;
    find(...queries: Query[]): DatabaseIterableIterator<T>;
    protected findQuery(query: Query): Promise<Set<number>>;
    insert(objects: T | T[]): Promise<void>;
    index(...fields: (string | IndexField)[]): Promise<void>;
    private static waitForReady;
    private static waitForClose;
    protected isIndexOutdated(): Promise<boolean>;
    protected getObjectFields(object: Record, position: number, fields: IndexField[]): ObjectField[];
    protected normalizeIndexFields(indexFields: (string | IndexField)[]): IndexField[];
    protected static getField(object: Record, field: string): any;
}
export interface Record {
    [field: string]: any;
}
export default Database;
