import JSONStore from './json-store';
import { Predicate } from './query';
declare class Index {
    filename: string;
    protected store: JSONStore<string>;
    protected maxHeight: number;
    protected logger: Console;
    constructor(filename: string);
    create(): Promise<void>;
    open(): Promise<void>;
    close(): Promise<void>;
    drop(): Promise<void>;
    readonly isOpen: boolean;
    lastModified(): Promise<Date>;
    beginTransaction(field: string): Promise<void>;
    endTransaction(field: string): Promise<void>;
    insert(objectFields: ObjectField | ObjectField[], cache?: IndexCache): Promise<void>;
    protected indexObjectField(objectField: ObjectField, head: IndexEntry, cache: IndexCache | undefined, inserts: IndexEntry[]): Promise<IndexEntry[]>;
    getFields(): Promise<IndexFieldInfo[]>;
    addFields(fields: IndexField[]): Promise<void>;
    protected lockHead(field: string, exclusive?: boolean, cache?: IndexCache): Promise<IndexEntry>;
    protected unlockHead(head: IndexEntry): Promise<void>;
    protected getRootEntry(cache?: IndexCache): Promise<IndexEntry>;
    protected insertEntry(entry: IndexEntry, cache?: IndexCache, position?: number): Promise<number>;
    protected getEntry(position: number, cache?: IndexCache): Promise<IndexEntry>;
    protected updateEntry(entry: IndexEntry): Promise<void>;
    find(field: string, predicate: Predicate<SkipListValue>): Promise<Set<number>>;
}
export declare class IndexError extends Error {
}
declare const enum SkipListValueType {
    Null = 0,
    Boolean = 1,
    Number = 2,
    String = 3
}
declare type SkipListValue = null | boolean | number | string;
declare class SkipListNode {
    levels: number[];
    value: SkipListValue;
    readonly type: SkipListValueType;
    constructor(encodedNode: string);
    constructor(levels: number[], value?: SkipListValue);
    readonly isDuplicate: boolean;
    next(level: number): number;
    encoded(): string;
    protected static decodeValue(type: SkipListValueType, value: string): SkipListValue;
}
declare class IndexEntry {
    position: number;
    pointer: number;
    link: number;
    node: SkipListNode;
    constructor(encodedEntry: string);
    constructor(pointer: number, node: SkipListNode);
    encoded(): string;
}
export interface ObjectField {
    name: string;
    value: any;
    position: number;
}
export interface IndexFieldInfo extends IndexField {
    tx?: number;
}
export interface IndexField {
    name: string;
    type?: string;
}
export interface IndexCache {
    get(key: number): IndexEntry | undefined;
    set(key: number, value: IndexEntry): void;
}
export default Index;
