"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const console_1 = require("console");
const fs = __importStar(require("fs"));
const stream_1 = require("stream");
const util = __importStar(require("util"));
const z85 = __importStar(require("z85"));
async function* readJSON(stream, parse = true) {
    let charCodes = [];
    let type = 0 /* Unknown */;
    let start = -1;
    let depth = 0;
    let inString = false;
    let escaping = false;
    let res;
    while (!(res = await stream.next()).done) {
        for (let i = 0; i < res.value.length; i += 2) {
            const codePoint = res.value[i + 1];
            if (start == -1) {
                if (codePoint == 32 /* Space */ || codePoint == 10 /* Newline */ ||
                    codePoint == 44 /* Comma */)
                    continue;
                else {
                    start = res.value[i];
                    if (codePoint == 123 /* LeftBrace */)
                        type = 2 /* Object */;
                    else if (codePoint == 34 /* Quote */)
                        type = 3 /* String */;
                    else if (codePoint == 91 /* LeftBracket */)
                        type = 1 /* Array */;
                }
            }
            if (parse) {
                if (codePoint > 0xFFFF) {
                    const code = codePoint - 0x10000;
                    charCodes.push(0xD800 | (code >> 10), 0xDC00 | (code & 0x3FF));
                }
                else
                    charCodes.push(codePoint);
            }
            const isStringQuote = codePoint == 34 /* Quote */ && !escaping;
            if (escaping)
                escaping = false;
            else if (codePoint == 92 /* Backslash */)
                escaping = true;
            if (isStringQuote)
                inString = !inString;
            if (inString && type != 3 /* String */)
                continue;
            switch (type) {
                case 1 /* Array */:
                    if (codePoint == 91 /* LeftBracket */)
                        ++depth;
                    else if (codePoint == 93 /* RightBracket */)
                        --depth;
                    break;
                case 2 /* Object */:
                    if (codePoint == 123 /* LeftBrace */)
                        ++depth;
                    else if (codePoint == 125 /* RightBrace */)
                        --depth;
                    break;
                case 3 /* String */:
                    if (isStringQuote)
                        depth = Number(!depth);
                    break;
                default:
                    if (codePoint == 32 /* Space */ || codePoint == 10 /* Newline */ ||
                        codePoint == 44 /* Comma */ || codePoint == 125 /* RightBrace */ ||
                        codePoint == 93 /* RightBracket */) {
                        --depth;
                        if (parse)
                            charCodes.pop();
                    }
                    else if (!depth)
                        ++depth;
            }
            if (!depth) {
                const length = res.value[i] - start + Number(type != 0 /* Unknown */);
                const result = { start, length };
                if (parse)
                    result.value = JSON.parse(String.fromCharCode.apply(null, charCodes));
                yield result;
                // Reset
                charCodes = [];
                type = 0 /* Unknown */;
                start = -1;
            }
        }
    }
}
exports.readJSON = readJSON;
function findJSONfield(text, field) {
    let prevChar;
    let inString = false;
    let depth = 0;
    let str = '';
    // TODO - review this code especially depth
    for (let i = 0; i < text.length; i++) {
        const char = text[i];
        if (char == ' ' || char == '\n')
            continue;
        const isStringQuote = char == '"' && prevChar != '\\';
        prevChar = char;
        if (isStringQuote) {
            inString = !inString;
            if (inString)
                str = '';
        }
        if (inString) {
            if (!isStringQuote)
                str += char;
            continue;
        }
        if (char == '}')
            --depth;
        else if (char == '{')
            ++depth;
        if (depth > 1)
            continue;
        // TODO - ensure only one object is searched
        if (char == ':' && str == field)
            return i + 1;
    }
    return;
}
exports.findJSONfield = findJSONfield;
function utf8codepoint(buf, i = 0) {
    const c = buf[i];
    switch (c >> 4) {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
        case 7:
            // 0xxxxxxx
            return c;
        case 12:
        case 13:
            // 110xxxxx 10xxxxxx
            return ((c & 0x1F) << 6) | (buf[i + 1] & 0x3F);
        case 14:
            // 1110xxxx 10xxxxxx 10xxxxxx
            return (((c & 0x0F) << 12) |
                ((buf[i + 1] & 0x3F) << 6) |
                (buf[i + 2] & 0x3F));
        case 15:
            if (!(c & 0x8))
                // 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
                return (((c & 0x07) << 18) |
                    ((buf[i + 1] & 0x3F) << 12) |
                    ((buf[i + 2] & 0x3F) << 6) |
                    (buf[i + 3] & 0x3F));
        default:
            throw new Error('Invalid UTF-8');
    }
}
const fstat = util.promisify(fs.fstat);
const fread = util.promisify(fs.read);
async function* read(fd, position = 0, reverse = false, buffer) {
    if (position < 0)
        position += (await fstat(fd)).size;
    buffer = buffer || Buffer.alloc(1 << 12);
    const size = buffer.length;
    let length = size;
    let pos = reverse ? position - size + 1 : position;
    const chars = Array(size * 2);
    let bytesRead = 0;
    let charCount = 0;
    let continuing = null;
    while (true) {
        if (!continuing) {
            charCount = 0;
            length = Math.min(size, size + pos);
            pos = Math.max(pos, 0);
            ({ bytesRead } = await fread(fd, buffer, 0, length, pos));
            for (let i = 0; i < bytesRead;) {
                let index = reverse ? (bytesRead - i - 1) : i;
                let count = 0;
                if (reverse) {
                    count = 1;
                    while ((buffer[index] & 0xc0) == 0x80)
                        --index, ++count;
                }
                else {
                    for (let b = 7; (buffer[index] >> b) & 1; b--)
                        ++count;
                    count = count || 1;
                }
                // Handle UTF-8 characters split at buffer boundary
                if (index < 0 || index + count > size)
                    length -= reverse ? index + count : size - index;
                else {
                    chars[charCount++] = pos + index;
                    chars[charCount++] = utf8codepoint(buffer, index);
                }
                i += count;
            }
        }
        continuing = null;
        try {
            if (charCount == chars.length)
                yield chars;
            else
                yield chars.slice(0, charCount);
            // If we reached here, it means we yielded successfully and can update
            // the state for the next iteration.
            continuing = false;
            if (bytesRead < size || (reverse && !pos))
                break;
            pos += reverse ? -length : length;
        }
        finally {
            if (continuing == null) {
                // If we reached here, then we didn't yield because the generator ended
                // prematurely, so remember to yield in the next iteration.
                continuing = true;
                continue;
            }
        }
    }
}
exports.read = read;
// Allocating buffers is expensive, so preallocate one
const uint32Buffer = Buffer.alloc(4);
function z85EncodeAsUInt32(number, compact = false) {
    uint32Buffer.writeUInt32BE(number, 0);
    const encoded = z85.encode(uint32Buffer);
    return compact ? encoded.replace(/^0+/, '') : encoded;
}
exports.z85EncodeAsUInt32 = z85EncodeAsUInt32;
function z85DecodeAsUInt32(string, compact = false) {
    if (string.length > 5)
        throw new Error('Cannot decode string longer than 5 characters');
    const decoded = z85.decode(compact ? string.padStart(5, '0') : string);
    return decoded.readUInt32BE(0);
}
exports.z85DecodeAsUInt32 = z85DecodeAsUInt32;
const uintBuffer = Buffer.alloc(8);
function z85EncodeAsUInt(number, compact = false) {
    uintBuffer.writeUIntBE(number, 2, 6);
    const encoded = z85.encode(uintBuffer);
    return compact ? encoded.replace(/^0+/, '') : encoded.slice(2);
}
exports.z85EncodeAsUInt = z85EncodeAsUInt;
function z85DecodeAsUInt(string, compact = false) {
    if (!compact && string.length != 8)
        throw new Error('String must be 8 characters long');
    else if (string.length > 8)
        throw new Error('Cannot decode string longer than 8 characters');
    const decoded = z85.decode(string.padStart(10, '0'));
    return decoded.readUIntBE(2, 6);
}
exports.z85DecodeAsUInt = z85DecodeAsUInt;
const doubleBuffer = Buffer.alloc(8);
function z85EncodeAsDouble(number, compact = false) {
    doubleBuffer.writeDoubleBE(number, 0);
    const encoded = z85.encode(doubleBuffer);
    return compact ? encoded.replace(/0+$/, '') : encoded;
}
exports.z85EncodeAsDouble = z85EncodeAsDouble;
function z85DecodeAsDouble(string, compact = false) {
    if (string.length > 10)
        throw new Error('Cannot decode string longer than 5 characters');
    const decoded = z85.decode(compact ? string.padEnd(10, '0') : string);
    return decoded.readDoubleBE(0);
}
exports.z85DecodeAsDouble = z85DecodeAsDouble;
class Out extends stream_1.Writable {
    constructor(label) {
        super();
        this.label = label;
    }
    _write(chunk, _, callback) {
        process.stdout.write(`${this.label}: ${chunk}`);
        if (callback)
            callback(null);
    }
}
class Err extends stream_1.Writable {
    constructor(label) {
        super();
        this.label = label;
    }
    _write(chunk, _, callback) {
        process.stderr.write(`${this.label}: ${chunk}`);
        if (callback)
            callback(null);
    }
}
const dummyConsole = new console_1.Console(new stream_1.Writable);
function logger(name, log) {
    if (log == null)
        log = Boolean(process.env.DEBUG);
    return log ? new console_1.Console(new Out(name), new Err(name)) : dummyConsole;
}
exports.logger = logger;
//# sourceMappingURL=utils.js.map